# common plot tools

import plotly.express as px
import plotly.graph_objects as go
import plotly.io as pio
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
import matplotlib
from copy import deepcopy
import colorsys

def lighten_color(color, amount=0.5):
    """
    Lightens the given color by multiplying (1-luminosity) by the given amount.
    Input can be matplotlib color string, hex string, or RGB tuple.

    Examples:
    >> lighten_color('g', 0.3)
    >> lighten_color('#F034A3', 0.6)
    >> lighten_color((.3,.55,.1), 0.5)
    """
    import matplotlib.colors as mc
    import colorsys
    try:
        c = mc.cnames[color]
    except:
        c = color
    c = colorsys.rgb_to_hls(*mc.to_rgb(c))
    c = colorsys.hls_to_rgb(c[0], 1 - amount * (1 - c[1]), c[2])
    # turn rgb into hex
    return '#%02x%02x%02x' % tuple(int(c[i] * 255) for i in range(3))

showConfig = {
  'toImageButtonOptions': {
    'format': 'svg',
  }
}

pio.kaleido.scope.mathjax = None  # walkaround the strange box issue

paperTemplate = pio.templates['plotly_white']
# paperTemplate.layout.colorway = [lighten_color(c, 0.5) for c in px.colors.qualitative.Plotly]
paperTemplate.layout.colorway = px.colors.qualitative.Plotly
paperTemplate.layout.colorscale = {'sequential': px.colors.sequential.Agsunset}
paperTemplate.layout.font.family = 'Arial'
paperTemplate.layout.font.color = 'black'
paperTemplate.layout.titlefont.size = 15
paperTemplate.layout.title.x = 0.5
paperTemplate.layout.xaxis.title.standoff = 1
paperTemplate.layout.xaxis.titlefont.size = 13
paperTemplate.layout.yaxis.title.standoff = 1
paperTemplate.layout.yaxis.titlefont.size = 13
paperTemplate.layout.xaxis.tickfont.size = 12
paperTemplate.layout.yaxis.tickfont.size = 12
paperTemplate.layout.xaxis.tickangle = 35
paperTemplate.layout.coloraxis.colorbar.len = 0.7
paperTemplate.layout.coloraxis.colorbar.thickness = 10
paperTemplate.layout.coloraxis.colorbar.outlinewidth = 1
paperTemplate.layout.coloraxis.colorbar.outlinecolor = 'black'
paperTemplate.layout.coloraxis.colorbar.title.font.size = 12
paperTemplate.layout.coloraxis.colorbar.tickfont.size = 12
paperTemplate.layout.coloraxis.colorbar.xpad = 0
paperTemplate.layout.margin = dict(l=10, r=10, t=10, b=10)
paperTemplate.data.bar[0]['marker']['line']['color'] = '#000000'
paperTemplate.data.bar[0]['marker']['line']['width'] = 1
paperTemplate.data.pie[0]['marker']['line']['color'] = '#000000'
paperTemplate.data.pie[0]['marker']['line']['width'] = 1
paperTemplate.data.pie[0]['textinfo'] = 'label+percent'
paperTemplate.data.pie[0]['textfont']['size'] = 15

paperTemplateLighter = deepcopy(paperTemplate)
paperTemplateLighter.layout.colorway = [lighten_color(c, 0.5) for c in px.colors.qualitative.Plotly]


def plot_value_counts_pie(s: pd.Series, top=0, **kwargs):
    """
    Plot a pie chart of the value counts of a Series.

    Parameters
    ----------
    s : pd.Series
        Series to plot.
    top : int, optional
        Maximum number of observations to include in the plot.
    **kwargs
        Additional keyword arguments passed to plotly.express.pie.

    Returns
    -------
    fig : plotly.graph_objects.Figure
        Plotly figure.
    """
    vc = s.value_counts()
    if top > 0:
        vc = vc[:top]
    fig = px.pie(names=vc.index, values=vc.values, template=paperTemplateLighter, **kwargs)
    return fig


def plot_mini_hist(data: pd.Series, log_scale=False, xlim=None):
    fig, ax = plt.subplots(figsize=(3, 0.5))
    sns.kdeplot(data, ax=ax, log_scale=log_scale, fill=True, color="black")
    if log_scale:
        x_major = matplotlib.ticker.LogLocator(base = 10.0, numticks = 5)
        ax.xaxis.set_major_locator(x_major)
        x_minor = matplotlib.ticker.LogLocator(base = 10.0, subs = np.arange(1.0, 10.0) * 0.1, numticks = 10)
        ax.xaxis.set_minor_locator(x_minor)
    ax.set_xlabel("")
    ax.set_xlim(min(data) if xlim is None else xlim)
    ax.get_yaxis().set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['left'].set_visible(False)
    return fig